@@ -2,7 +2,22 @@
 # Copyright 2019 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Access a Berkeley DB database with the bsddb3 module."""
+"""Access a Berkeley DB database with the berkeleydb or bsddb3 modules.
+
+berkeleydb is available at Python 3.6 versions and later.
+bsddb3 is available at Python 3.9 versions and earlier.
+
+See jcea.es/programacion/pybsddb.htm for restrictions on combining bsddb3
+and berkeleydb with versions of Berkeley DB.
+
+OpenBSD 7.3 ports and packages provides Berkeley DB 4.6.21 and bsddb3-6.0.1
+which works on Python 3.9, but not Python 3.10 (the default version) or
+Python 3.11; which are the three Python ports provided.
+
+Berkeley DB 4.6 is not compatible with berkeleydb, but can be accessed by
+the _db_tkinter module, a sibling of _db (this module).
+
+"""
 import os
 from ast import literal_eval
 import bisect
@@ -34,12 +49,13 @@ from .segmentsize import SegmentSize
 # Did not bother about this until pylint with default settings gave
 # warnings.
 from . import cursor as _cursor
+from . import recordsetcursor
 from .recordset import (
     RecordsetSegmentBitarray,
     RecordsetSegmentInt,
     RecordsetSegmentList,
-    RecordsetCursor as _RecordsetCursor,
     RecordList,
+    FoundSet,
 )
 
 # DBenv parameter maxlocks may need setting on OpenBSD.
@@ -54,6 +70,9 @@ class DatabaseError(Exception):
 class Database(_database.Database):
     """Define file and record access methods."""
 
+    # Default checkpoint interval after commits.
+    _MINIMUM_CHECKPOINT_INTERVAL = 5
+
     class SegmentSizeError(Exception):
         """Raise when segment size in database is not in specification."""
 
@@ -68,14 +87,15 @@ class Database(_database.Database):
         **soak
     ):
         """Initialize data structures."""
+        del soak
         if folder is not None:
             try:
                 path = os.path.abspath(folder)
-            except:
+            except Exception as exc:
                 msg = " ".join(
                     ["Database folder name", str(folder), "is not valid"]
                 )
-                raise DatabaseError(msg)
+                raise DatabaseError(msg) from exc
         else:
             path = None
         if not isinstance(specification, filespec.FileSpec):
@@ -104,7 +124,6 @@ class Database(_database.Database):
         self._dbe = None
         self.segment_table = {}
         self.ebm_control = {}
-        self.ebm_segment_count = {}
 
         # Set to value read from database on attempting to open database if
         # different from segment_size_bytes.
@@ -120,8 +139,8 @@ class Database(_database.Database):
         # same name as the database.
         # The databases created in an empty home_directory from a FileSpec are
         # placed in separate files if file_per_database is True.
-        self._file_per_database = file_per_database
-        self._initial_file_per_database = file_per_database
+        self._file_per_database = bool(file_per_database)
+        self._initial_file_per_database = bool(file_per_database)
 
     def _validate_segment_size_bytes(self, segment_size_bytes):
         if segment_size_bytes is None:
@@ -141,14 +160,14 @@ class Database(_database.Database):
         if self.dbtxn is not None:
             self.dbtxn.abort()
             self.dbtxn = None
-            self.dbenv.txn_checkpoint(5)
+            self.dbenv.txn_checkpoint(self._MINIMUM_CHECKPOINT_INTERVAL)
 
     def commit(self):
         """Commit the active transaction and remove binding to txn object."""
         if self.dbtxn is not None:
             self.dbtxn.commit()
             self.dbtxn = None
-            self.dbenv.txn_checkpoint(5)
+            self.dbenv.txn_checkpoint(self._MINIMUM_CHECKPOINT_INTERVAL)
 
     def file_name_for_database(self, database):
         """Return filename for database.
@@ -186,7 +205,7 @@ class Database(_database.Database):
                         self.database_file, dbname=name, flags=dbe.DB_RDONLY
                     )
                     all_in_one.add(name)
-                except:
+                except Exception:
                     pass
                 finally:
                     dbo.close()
@@ -198,7 +217,7 @@ class Database(_database.Database):
                         flags=dbe.DB_RDONLY,
                     )
                     one_per_database.add(name)
-                except:
+                except Exception:
                     pass
                 finally:
                     dbo.close()
@@ -270,6 +289,29 @@ class Database(_database.Database):
         gbytes = self.environment.get("gbytes", 0)
         bytes_ = self.environment.get("bytes", 0)
         self.dbenv = dbe.DBEnv()
+        # bsddb3-6.0.1 does not have the log_set_config() method.
+        # bsddb3-6.1.0 does have this method.
+        # OpenBSD 7.3 has Berkeley DB 4.6 which is compatible with
+        # bsddb3-6.0.1 but not bsddb3-6.1.0 but most other BSDs which use
+        # an old Berkeley DB (licence reasons assumed) have version 5.3.28
+        # which is compatible with bsddb3-6.1.0 but not bsddb3-6.0.1.
+        # bsddb3-6.0.1 documentation advertises the log_set_config() method
+        # but the Berkeley DB 4.6.21 documentation does not.
+        # bsddb3-6.1.0 documentation advertises the log_set_config() method
+        # as does the Berkeley DB 5.3.28 documentation.
+        # The sibling _db_tkinter module cannot call the log_set_config()
+        # method and runs a subprocesss to do a db archive to remove old
+        # log files at some useful points: it expects the utility to be
+        # '/usr/local/bin/db4_archive', being the OpenBSD name.
+        # Above ignores recommendation to use berkeleydb, not bsddb3, at
+        # Python 3.6 and later.
+        try:
+            self.dbenv.log_set_config(dbe.DB_LOG_AUTO_REMOVE, 1)
+        except AttributeError as exc:
+            if not _openbsd_platform:
+                raise
+            if "'log_set_config'" not in str(exc):
+                raise
         if gbytes or bytes_:
             self.dbenv.set_cachesize(gbytes, bytes_)
         if self.home_directory is not None:
@@ -323,10 +365,22 @@ class Database(_database.Database):
         # FIDE longest possible game, at nearly 6000 moves, might be a problem.
         # maxlocks is added to the things taken from self.environment but is
         # only applied if non-zero and OS is OpenBSD.
+        # Dated 28 December 2022.
+        # Delete the DB_CONFIG file in <home> directory if it exists to
+        # avoid mis-use.  Also the possibility the _db_tkinter module did
+        # not get to delete it after legitimate use in that module.
+        if self.home_directory is not None:
+            try:
+                os.remove(os.path.join(self.home_directory, "DB_CONFIG"))
+            except FileNotFoundError:
+                pass
         if _openbsd_platform:
             maxlocks = self.environment.get("maxlocks", 0)
+            maxobjects = self.environment.get("maxobjects", 0)
             if maxlocks:
                 self.dbenv.set_lk_max_locks(maxlocks)
+            if maxobjects:
+                self.dbenv.set_lk_max_objects(maxobjects)
 
         self.dbenv.open(self.home_directory, self.environment_flags(dbe))
         if files is None:
@@ -349,9 +403,9 @@ class Database(_database.Database):
             if file not in files:
                 continue
             fields = specification[SECONDARY]
-            self.table[file] = [dbe.DB(self.dbenv)]
+            self.table[file] = dbe.DB(self.dbenv)
             try:
-                self.table[file][0].open(
+                self.table[file].open(
                     self.file_name_for_database(file),
                     dbname=file,
                     dbtype=dbe.DB_RECNO,
@@ -391,10 +445,10 @@ class Database(_database.Database):
                 else:
                     access_method = dbe.DB_BTREE
                 secondary = SUBFILE_DELIMITER.join((file, field))
-                self.table[secondary] = [dbe.DB(self.dbenv)]
+                self.table[secondary] = dbe.DB(self.dbenv)
                 try:
-                    self.table[secondary][0].set_flags(dbe.DB_DUPSORT)
-                    self.table[secondary][0].open(
+                    self.table[secondary].set_flags(dbe.DB_DUPSORT)
+                    self.table[secondary].open(
                         self.file_name_for_database(secondary),
                         dbname=secondary,
                         dbtype=access_method,
@@ -402,7 +456,7 @@ class Database(_database.Database):
                         txn=self.dbtxn,
                     )
                 except dbe.DBInvalidArgError as exc:
-                    if not str(exc) == secondary.join(
+                    if str(exc) != secondary.join(
                         (
                             "(22, 'Invalid argument -- ",
                             ": unexpected file type or format')",
@@ -414,10 +468,10 @@ class Database(_database.Database):
 
                     # Accept existing DB_BTREE database if DB_HASH was in the
                     # supplied specification for database.
-                    self.table[secondary] = [dbe.DB(self.dbenv)]
+                    self.table[secondary] = dbe.DB(self.dbenv)
                     try:
-                        self.table[secondary][0].set_flags(dbe.DB_DUPSORT)
-                        self.table[secondary][0].open(
+                        self.table[secondary].set_flags(dbe.DB_DUPSORT)
+                        self.table[secondary].open(
                             self.file_name_for_database(secondary),
                             dbname=secondary,
                             dbtype=dbe.DB_BTREE,
@@ -460,7 +514,8 @@ class Database(_database.Database):
     def checkpoint_before_close_dbenv(self):
         """Do a checkpoint call."""
         # Rely on environment_flags() call for transaction state.
-        self.dbenv.txn_checkpoint()
+        if self.dbtxn is not None:
+            self.dbenv.txn_checkpoint()
 
     def close_database_contexts(self, files=None):
         """Close files in database.
@@ -475,12 +530,12 @@ class Database(_database.Database):
         all the DB objects were created in the context of the DBEnv object.
 
         """
+        del files
         self._dbe = None
         for file, specification in self.specification.items():
             if file in self.table:
                 if self.table[file] is not None:
-                    for dbo in self.table[file]:
-                        dbo.close()
+                    self.table[file].close()
                     self.table[file] = None
             if file in self.segment_table:
                 if self.segment_table[file] is not None:
@@ -494,8 +549,7 @@ class Database(_database.Database):
             for field in specification[SECONDARY]:
                 secondary = SUBFILE_DELIMITER.join((file, field))
                 if secondary in self.table:
-                    for dbo in self.table[secondary]:
-                        dbo.close()
+                    self.table[secondary].close()
                     self.table[secondary] = None
         for k, dbo in self.table.items():
             if dbo is not None:
@@ -508,7 +562,6 @@ class Database(_database.Database):
             self.table = {}
             self.segment_table = {}
             self.ebm_control = {}
-            self.ebm_segment_count = {}
         self.segment_size_bytes = self._initial_segment_size_bytes
 
     def close_database(self):
@@ -525,8 +578,8 @@ class Database(_database.Database):
         """Insert key, or replace key, in table for file using value."""
         assert file in self.specification
         if key is None:
-            return self.table[file][0].append(value.encode(), txn=self.dbtxn)
-        self.table[file][0].put(key, value.encode(), txn=self.dbtxn)
+            return self.table[file].append(value.encode(), txn=self.dbtxn)
+        self.table[file].put(key, value.encode(), txn=self.dbtxn)
         return None
 
     def replace(self, file, key, oldvalue, newvalue):
@@ -534,18 +587,20 @@ class Database(_database.Database):
 
         oldvalue is ignored in _sqlite version of replace() method.
         """
+        del oldvalue
         assert file in self.specification
-        self.table[file][0].put(key, newvalue.encode(), txn=self.dbtxn)
+        self.table[file].put(key, newvalue.encode(), txn=self.dbtxn)
 
     def delete(self, file, key, value):
         """Delete key from table for file.
 
         value is ignored in _db version of delete() method.
         """
+        del value
         assert file in self.specification
         try:
-            self.table[file][0].delete(key, txn=self.dbtxn)
-        except:
+            self.table[file].delete(key, txn=self.dbtxn)
+        except Exception:
             pass
 
     def get_primary_record(self, file, key):
@@ -553,7 +608,7 @@ class Database(_database.Database):
         assert file in self.specification
         if key is None:
             return None
-        record = self.table[file][0].get(key, txn=self.dbtxn)
+        record = self.table[file].get(key, txn=self.dbtxn)
         if record is None:
             return None
         return key, record.decode()
@@ -612,7 +667,6 @@ class Database(_database.Database):
 
             lfrns = ebmc.read_exists_segment(segment_number, self.dbtxn)
             if lfrns is None:
-
                 # Segment does not exist now.
                 ebmc.freed_record_number_pages.remove(segment_number)
                 continue
@@ -652,12 +706,12 @@ class Database(_database.Database):
         note_freed_record_number_segment.  A successful record deletion
         passes this test.
         """
+        del record_number_in_segment
         try:
             high_segment = divmod(high_record[0], SegmentSize.db_segment_size)[
                 0
             ]
         except TypeError:
-
             # Implies attempt to delete record from empty database.
             # The delete method will have raised an exception if appropriate.
             return
@@ -741,7 +795,7 @@ class Database(_database.Database):
     # Only used in one place, and it is extra work to get the data in_nosql.
     def get_high_record(self, file):
         """Return the high existing record number in table for file."""
-        cursor = self.table[file][0].cursor(txn=self.dbtxn)
+        cursor = self.table[file].cursor(txn=self.dbtxn)
         try:
             return cursor.last()
         finally:
@@ -774,13 +828,12 @@ class Database(_database.Database):
         """
         key = self.encode_record_selector(key)
         secondary = SUBFILE_DELIMITER.join((file, field))
-        cursor = self.table[secondary][0].cursor(txn=self.dbtxn)
+        cursor = self.table[secondary].cursor(txn=self.dbtxn)
         try:
             record = cursor.set_range(key)
             while record:
                 record_key, value = record
                 if record_key != key:
-
                     # No index entry for key.
                     cursor.put(
                         key,
@@ -799,7 +852,6 @@ class Database(_database.Database):
                     record = cursor.next_dup()
                     continue
                 if segment_number > segment:
-
                     # No index entry for key in this segment.
                     cursor.put(
                         key,
@@ -949,13 +1001,12 @@ class Database(_database.Database):
         """
         key = self.encode_record_selector(key)
         secondary = SUBFILE_DELIMITER.join((file, field))
-        cursor = self.table[secondary][0].cursor(txn=self.dbtxn)
+        cursor = self.table[secondary].cursor(txn=self.dbtxn)
         try:
             record = cursor.set_range(key)
             while record:
                 record_key, value = record
                 if record_key != key:
-
                     # Assume that multiple requests to delete an index value
                     # have been made for a record.  The segment_put method uses
                     # sets to avoid adding multiple entries.  Consider using
@@ -1089,7 +1140,8 @@ class Database(_database.Database):
         segment_record = self.segment_table[file].get(
             int.from_bytes(
                 segment_reference[SEGMENT_HEADER_LENGTH:], byteorder="big"
-            )
+            ),
+            txn=self.dbtxn,
         )
         if segment_record is None:
             raise DatabaseError("Segment record missing")
@@ -1107,8 +1159,8 @@ class Database(_database.Database):
 
     def find_values(self, valuespec, file):
         """Yield values in range defined in valuespec in index named file."""
-        cursor = self.table[SUBFILE_DELIMITER.join((file, valuespec.field))][
-            0
+        cursor = self.table[
+            SUBFILE_DELIMITER.join((file, valuespec.field))
         ].cursor(txn=self.dbtxn)
         try:
             if valuespec.above_value and valuespec.below_value:
@@ -1294,7 +1346,7 @@ class Database(_database.Database):
                     recordlist[segment_start][(segment_start, i)] = False
             if final_segment is not None:
                 for i in range(
-                    final_segment * 8 + end_byte, (final_segment + 1) * 8
+                    final_segment * 8 + end_byte + 1, (final_segment + 1) * 8
                 ):
                     recordlist[segment_end][(segment_end, i)] = False
         finally:
@@ -1308,7 +1360,6 @@ class Database(_database.Database):
         try:
             record = cursor.first()
             while record:
-
                 # The keys in self.ebm_control[file].ebm_table are always
                 # 'segment + 1' because automatically allocated RECNO keys
                 # start at 1 in an empty table and segment numbers start at 0.
@@ -1339,7 +1390,8 @@ class Database(_database.Database):
             segment_record = self.segment_table[recordset.dbset].get(
                 int.from_bytes(
                     reference[SEGMENT_HEADER_LENGTH:], byteorder="big"
-                )
+                ),
+                txn=self.dbtxn,
             )
             if len(segment_record) == SegmentSize.db_segment_size_bytes:
                 segment = RecordsetSegmentBitarray(
@@ -1363,7 +1415,7 @@ class Database(_database.Database):
         if keylike is None:
             return recordlist
         pattern = b".*?" + keylike
-        cursor = self.table[SUBFILE_DELIMITER.join((file, field))][0].cursor(
+        cursor = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
             txn=self.dbtxn
         )
         try:
@@ -1380,7 +1432,7 @@ class Database(_database.Database):
     def recordlist_key(self, file, field, key=None, cache_size=1):
         """Return RecordList on file containing records for field with key."""
         recordlist = RecordList(dbhome=self, dbset=file, cache_size=cache_size)
-        cursor = self.table[SUBFILE_DELIMITER.join((file, field))][0].cursor(
+        cursor = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
             txn=self.dbtxn
         )
         try:
@@ -1405,7 +1457,7 @@ class Database(_database.Database):
         recordlist = RecordList(dbhome=self, dbset=file, cache_size=cache_size)
         if keystart is None:
             return recordlist
-        cursor = self.table[SUBFILE_DELIMITER.join((file, field))][0].cursor(
+        cursor = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
             txn=self.dbtxn
         )
         try:
@@ -1431,7 +1483,7 @@ class Database(_database.Database):
         if le and lt:
             raise DatabaseError("Both 'le' and 'lt' given in key range")
         recordlist = RecordList(dbhome=self, dbset=file, cache_size=cache_size)
-        cursor = self.table[SUBFILE_DELIMITER.join((file, field))][0].cursor(
+        cursor = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
             txn=self.dbtxn
         )
         try:
@@ -1467,7 +1519,7 @@ class Database(_database.Database):
     def recordlist_all(self, file, field, cache_size=1):
         """Return RecordList on file containing records for field."""
         recordlist = RecordList(dbhome=self, dbset=file, cache_size=cache_size)
-        cursor = self.table[SUBFILE_DELIMITER.join((file, field))][0].cursor(
+        cursor = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
             txn=self.dbtxn
         )
         try:
@@ -1490,11 +1542,10 @@ class Database(_database.Database):
         is deleted.
 
         """
-        cursor = self.table[SUBFILE_DELIMITER.join((file, field))][0].cursor(
+        cursor = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
             txn=self.dbtxn
         )
         try:
-
             # Delete segment records.
             record = cursor.set_range(key)
             while record:
@@ -1517,7 +1568,7 @@ class Database(_database.Database):
             # Delete segment references.
             # try:
             #    self.table[SUBFILE_DELIMITER.join((file, field))
-            #               ][0].delete(key, txn=self.dbtxn)
+            #               ].delete(key, txn=self.dbtxn)
             # except self._dbe.DBNotFoundError:
             #    pass
 
@@ -1540,7 +1591,7 @@ class Database(_database.Database):
         # an exception.
         #
         try:
-            self.table[SUBFILE_DELIMITER.join((file, field))][0].delete(
+            self.table[SUBFILE_DELIMITER.join((file, field))].delete(
                 key, txn=self.dbtxn
             )
         except self._dbe.DBNotFoundError:
@@ -1554,7 +1605,7 @@ class Database(_database.Database):
         # Delete existing segments for key
         self.unfile_records_under(file, field, key)
 
-        cursor = self.table[SUBFILE_DELIMITER.join((file, field))][0].cursor(
+        cursor = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
             txn=self.dbtxn
         )
         try:
@@ -1597,23 +1648,27 @@ class Database(_database.Database):
         finally:
             cursor.close()
 
-    def database_cursor(self, file, field, keyrange=None):
+    def database_cursor(self, file, field, keyrange=None, recordset=None):
         """Create and return a cursor on DB() for (file, field).
 
         keyrange is an addition for DPT. It may yet be removed.
+        recordset must be an instance of RecordList or FoundSet, or None.
 
         """
         assert file in self.specification
+        if recordset is not None:
+            assert isinstance(recordset, (RecordList, FoundSet))
+            return recordset.create_recordsetbase_cursor(internalcursor=True)
         if file == field:
             return CursorPrimary(
-                self.table[file][0],
+                self.table[file],
                 keyrange=keyrange,
                 transaction=self.dbtxn,
                 ebm=self.ebm_control[file].ebm_table,
                 engine=self._dbe,
             )
         return CursorSecondary(
-            self.table[SUBFILE_DELIMITER.join((file, field))][0],
+            self.table[SUBFILE_DELIMITER.join((file, field))],
             keyrange=keyrange,
             transaction=self.dbtxn,
             segment=self.segment_table[file],
@@ -1624,19 +1679,19 @@ class Database(_database.Database):
         return RecordsetCursor(
             recordset,
             transaction=self.dbtxn,
-            database=self.table[recordset.dbset][0],
+            database=self.table[recordset.dbset],
         )
 
     # Comment in chess_ui for make_position_analysis_data_source method, only
     # call, suggests is_database_file_active should not be needed.
     def is_database_file_active(self, file):
         """Return True if the DB object for file exists."""
-        return self.table[file][0] is not None
+        return self.table[file] is not None
 
     def get_table_connection(self, file):
         """Return main DB object for file."""
         if self.dbenv:
-            return self.table[file][0]
+            return self.table[file]
         return None
 
     def do_database_task(
@@ -1694,6 +1749,18 @@ class Database(_database.Database):
         finally:
             db.close_database()
 
+    def _generate_database_file_name(self, name):
+        """Extend, return path to Berkeley DB file for name.
+
+        Delegate to superclass if all databases are in one file.
+
+        Otherwise return stem of name for databases associated with name.
+
+        """
+        if not self._file_per_database:
+            return super()._generate_database_file_name(name)
+        return os.path.join(self.home_directory, name)
+
 
 class Cursor(_cursor.Cursor):
     """Define a cursor on the underlying database engine dbset.
@@ -1718,6 +1785,7 @@ class Cursor(_cursor.Cursor):
 
     def __init__(self, dbset, keyrange=None, transaction=None, **kargs):
         """Define a cursor on the underlying database engine dbset."""
+        del keyrange, kargs
         super().__init__(dbset)
         self._transaction = transaction
         self._cursor = dbset.cursor(txn=transaction)
@@ -1775,6 +1843,7 @@ class CursorPrimary(Cursor):
 
     def get_position_of_record(self, record=None):
         """Return position of record in file or 0 (zero)."""
+        # record keys are 1-based but segment_numbers are 0-based.
         if record is None:
             return 0
         segment_number, record_number = divmod(
@@ -1791,16 +1860,19 @@ class CursorPrimary(Cursor):
         segment_ebm = Bitarray()
         segment_ebm.frombytes(segment)
         try:
-            position += segment_ebm.search(SINGLEBIT).index(record_number)
+            position += segment_ebm.search(SINGLEBIT).index(record_number) + 1
         except ValueError:
-            position += bisect.bisect_left(
-                record_number, segment_ebm.search(SINGLEBIT)
+            position += (
+                bisect.bisect_left(
+                    segment_ebm.search(SINGLEBIT), record_number
+                )
+                + 1
             )
         return position
 
     def get_record_at_position(self, position=None):
         """Return record for positionth record in file or None."""
-        if position is None:
+        if not position:  # Include position 0 in this case.
             return None
         count = 0
         abspos = abs(position)
@@ -1831,9 +1903,9 @@ class CursorPrimary(Cursor):
                         count += ebm_count
                         record = ebm_cursor.next()
                         continue
-                    recno = segment_ebm.search(SINGLEBIT)[position - count] + (
-                        (record[0] - 1) * SegmentSize.db_segment_size
-                    )
+                    recno = segment_ebm.search(SINGLEBIT)[
+                        position - count - 1
+                    ] + ((record[0] - 1) * SegmentSize.db_segment_size)
                     # ebm_cursor.close()
                     return self._decode_record(self._cursor.set(recno))
         finally:
@@ -1974,14 +2046,22 @@ class CursorSecondary(Cursor):
             value, SegmentSize.db_segment_size
         )
 
-        # Define lambdas to handle presence or absence of partial key
-        low = lambda jkey, recordkey: jkey < recordkey
-        if not self.get_partial():
-            high = lambda jkey, recordkey: jkey > recordkey
-        else:
-            high = lambda jkey, partial: not jkey.startswith(partial)
+        # Define functions to handle presence or absence of partial key.
 
-        # Get position of record relative to start point
+        def low(jkey, recordkey):
+            return jkey < recordkey
+
+        if not self.get_partial():
+
+            def high(jkey, recordkey):
+                return jkey > recordkey
+
+        else:
+
+            def high(jkey, partial):
+                return not jkey.startswith(partial)
+
+        # Get position of record relative to start point.
         position = 0
         if not self.get_partial():
             j = self._cursor.first()
@@ -2020,7 +2100,7 @@ class CursorSecondary(Cursor):
             return None
 
         # Start at first or last record whichever is likely closer to position
-        # and define lambdas to handle presence or absence of partial key.
+        # and define functions to handle presence or absence of partial key.
         if not self.get_partial():
             get_partial = self.get_partial
         else:
@@ -2028,15 +2108,28 @@ class CursorSecondary(Cursor):
         if position < 0:
             step = self._cursor.prev
             if not self.get_partial():
-                start = lambda partial: self._cursor.last()
+
+                def start(partial):
+                    del partial
+                    return self._cursor.last()
+
             else:
-                start = lambda partial: self._last_partial(partial)
+
+                def start(partial):
+                    return self._last_partial(partial)
+
         else:
             step = self._cursor.next
             if not self.get_partial():
-                start = lambda partial: self._cursor.first()
+
+                def start(partial):
+                    del partial
+                    return self._cursor.first()
+
             else:
-                start = lambda partial: self._first_partial(partial)
+
+                def start(partial):
+                    return self._first_partial(partial)
 
         # Get record at position relative to start point.
         count = 0
@@ -2077,6 +2170,7 @@ class CursorSecondary(Cursor):
                 if record_number is not None:
                     return record[0].decode(), record_number
                 break
+        return None
 
     def last(self):
         """Return last record taking partial key into account."""
@@ -2235,7 +2329,8 @@ class CursorSecondary(Cursor):
         return record
 
     def _set_both(self, key, value):
-        segment, record_number = divmod(value, SegmentSize.db_segment_size)
+        # segment, record_number = divmod(value, SegmentSize.db_segment_size)
+        segment = divmod(value, SegmentSize.db_segment_size)[0]
         cursor = self._dbset.cursor(txn=self._transaction)
         try:
             record = cursor.set_range(key)
@@ -2297,6 +2392,7 @@ class CursorSecondary(Cursor):
             return None
         if record[0].startswith(partial_key):
             return self.set_current_segment(*record).last()
+        return None
 
     def refresh_recordset(self, instance=None):
         """Refresh records for datagrid access after database update.
@@ -2323,7 +2419,7 @@ class CursorSecondary(Cursor):
         return record[1]
 
 
-class RecordsetCursor(_RecordsetCursor):
+class RecordsetCursor(recordsetcursor.RecordsetCursor):
     """Add _get_record method and tranasction support to RecordsetCursor."""
 
     def __init__(self, recordset, transaction=None, database=None, **kargs):
@@ -2334,6 +2430,7 @@ class RecordsetCursor(_RecordsetCursor):
         kargs absorbs arguments relevant to other database engines.
 
         """
+        del kargs
         super().__init__(recordset)
         self._transaction = transaction
         self._database = database
@@ -2396,14 +2493,7 @@ class ExistenceBitmapControl(_database.ExistenceBitmapControl):
             raise
 
     def read_exists_segment(self, segment_number, dbtxn):
-        """Return existence bitmap for segment_number in database dbenv.
-
-        get_ebm_segment returns the record containing the existence bitmap,
-        read_exists_segment calls get_ebm_segment to get the bitmap record,
-        converts it to a bitmap, and then returns the bitmap.
-
-        """
-        # Return existence bit map for segment_number.
+        """Return existence bitmap for segment_number in database dbenv."""
         # record keys are 1-based but segment_numbers are 0-based.
         ebm = Bitarray()
         ebm.frombytes(self.ebm_table.get(segment_number + 1, txn=dbtxn))
