@@ -1,18 +1,16 @@
-# dbdu.py
+# _dbdu.py
 # Copyright (c) 2019 Roger Marsh
 # Licence: See LICENCE (BSD licence)
 
-"""Access a Berkeley DB database with the bsddb3 module."""
-import heapq
-import collections
-
-from .bytebit import Bitarray
+"""Access a Berkeley DB database with the berkeleydb or bsddb3 modules."""
 from .constants import (
     SECONDARY,
     # ACCESS_METHOD,
     # HASH,
     SUBFILE_DELIMITER,
     SEGMENT_HEADER_LENGTH,
+    EXISTING_SEGMENT_REFERENCE,
+    NEW_SEGMENT_CONTENT,
 )
 from .segmentsize import SegmentSize
 from .recordset import (
@@ -40,6 +38,9 @@ class Database(_databasedu.Database):
     segment.
     """
 
+    # Always checkpoint after commit in deferred update.
+    _MINIMUM_CHECKPOINT_INTERVAL = 0
+
     def __init__(self, *a, **kw):
         """Extend and initialize deferred update data structures."""
         super().__init__(*a, **kw)
@@ -51,40 +52,15 @@ class Database(_databasedu.Database):
         self.value_segments = {}  # was values in secondarydu.Secondary
         self._int_to_bytes = None
 
-    def database_cursor(self, file, field, keyrange=None):
+    def database_cursor(self, file, field, keyrange=None, recordset=None):
         """Not implemented for deferred update."""
         raise DatabaseError("database_cursor not implemented")
 
-    # Deferred updates are non-transactional in Berkeley DB.
-    def environment_flags(self, dbe):
-        """Return environment flags for deferred update."""
-        return (
-            dbe.DB_CREATE
-            |
-            # dbe.DB_RECOVER |
-            dbe.DB_INIT_MPOOL
-            | dbe.DB_INIT_LOCK
-            |
-            # dbe.DB_INIT_LOG |
-            # dbe.DB_INIT_TXN |
-            dbe.DB_PRIVATE
-        )
-
-    def checkpoint_before_close_dbenv(self):
-        """Do nothing.  Deferred updates are non-transactional."""
-        # Most calls of txn_checkpoint() are conditional on self.dbtxn, but the
-        # call when closing the database does not check for a transaction.
-        # Rely on environment_flags() call for transaction state.
-
-    def start_transaction(self):
-        """Do not start transaction in deferred update mode."""
-        self.dbtxn = None
-
     def do_final_segment_deferred_updates(self):
         """Do deferred updates for partially filled final segment."""
         # Write the final deferred segment database for each index
         for file in self.existence_bit_maps:
-            dbc = self.table[file][0].cursor(txn=self.dbtxn)
+            dbc = self.table[file].cursor(txn=self.dbtxn)
             try:
                 segment, record_number = divmod(
                     dbc.last()[0], SegmentSize.db_segment_size
@@ -95,20 +71,17 @@ class Database(_databasedu.Database):
                 continue
             finally:
                 dbc.close()
-            self.write_existence_bit_map(file, segment)
+            self._write_existence_bit_map(file, segment)
             for secondary in self.specification[file][SECONDARY]:
                 self.sort_and_write(file, secondary, segment)
                 self.merge(file, secondary)
 
     def set_defer_update(self):
         """Prepare to do deferred update run."""
-        self._int_to_bytes = [
-            n.to_bytes(2, byteorder="big")
-            for n in range(SegmentSize.db_segment_size)
-        ]
+        self.set_int_to_bytes_lookup(lookup=True)
         self.start_transaction()
         for file in self.specification:
-            dbc = self.table[file][0].cursor()
+            dbc = self.table[file].cursor(txn=self.dbtxn)
             try:
                 high_record = dbc.last()
             finally:
@@ -127,16 +100,18 @@ class Database(_databasedu.Database):
 
     def unset_defer_update(self):
         """Unset deferred update for db DBs. Default all."""
-        self._int_to_bytes = None
+        self.set_int_to_bytes_lookup(lookup=False)
         for file in self.specification:
             self.high_segment[file] = None
             self.first_chunk[file] = None
         self.commit()
 
-    def write_existence_bit_map(self, file, segment):
+    def _write_existence_bit_map(self, file, segment):
         """Write the existence bit map for segment."""
         self.ebm_control[file].ebm_table.put(
-            segment + 1, self.existence_bit_maps[file][segment].tobytes()
+            segment + 1,
+            self.existence_bit_maps[file][segment].tobytes(),
+            txn=self.dbtxn,
         )
 
     def _sort_and_write_high_or_chunk(
@@ -152,16 +127,15 @@ class Database(_databasedu.Database):
         # To verify path coverage uncomment the '_path_marker' code.
         # self._path_marker = set()
         segkeys = sorted(segvalues)
-        cursor_high = self.table[SUBFILE_DELIMITER.join((file, field))][
-            -1
-        ].cursor(txn=self.dbtxn)
+        cursor_high = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
+            txn=self.dbtxn
+        )
         try:
             for skey in segkeys:
                 k = skey.encode()
 
                 # Get high existing segment for value.
                 if not cursor_high.set(k):
-
                     # No segments for this index value.
                     # self._path_marker.add('p1')
                     continue
@@ -173,7 +147,6 @@ class Database(_databasedu.Database):
                     # self._path_marker.add('p2b')
                     segref = cursor_high.prev()[1]
                 if segment != int.from_bytes(segref[:4], byteorder="big"):
-
                     # No records exist in high segment for this index
                     # value.
                     # self._path_marker.add('p3')
@@ -205,7 +178,9 @@ class Database(_databasedu.Database):
                     if isinstance(current_segment, RecordsetSegmentList):
                         # self._path_marker.add('p5a-a')
                         self.segment_table[file].put(
-                            int.from_bytes(segref[-4:], "big"), seg.tobytes()
+                            int.from_bytes(segref[-4:], "big"),
+                            seg.tobytes(),
+                            txn=self.dbtxn,
                         )
                         cursor_high.delete()
                         cursor_high.put(
@@ -221,7 +196,12 @@ class Database(_databasedu.Database):
                         )
                     elif isinstance(current_segment, RecordsetSegmentInt):
                         # self._path_marker.add('p5a-b')
-                        srn = self.segment_table[file].append(seg.tobytes())
+                        srn = self.segment_table[file].append(
+                            seg.tobytes(), txn=self.dbtxn
+                        )
+                        # Why not use cursor_high throughout this method?
+                        # Then why not use DB_CURRENT and remove the delete()?
+                        cursor_high.delete()
                         cursor_new.put(
                             k,
                             b"".join(
@@ -236,7 +216,9 @@ class Database(_databasedu.Database):
                     else:
                         # self._path_marker.add('p5a-c')
                         self.segment_table[file].put(
-                            int.from_bytes(segref[-4:], "big"), seg.tobytes()
+                            int.from_bytes(segref[-4:], "big"),
+                            seg.tobytes(),
+                            txn=self.dbtxn,
                         )
                         cursor_high.delete()
                         cursor_high.put(
@@ -254,7 +236,12 @@ class Database(_databasedu.Database):
                     # self._path_marker.add('p5b')
                     if isinstance(current_segment, RecordsetSegmentInt):
                         # self._path_marker.add('p5b-a')
-                        srn = self.segment_table[file].append(seg.tobytes())
+                        srn = self.segment_table[file].append(
+                            seg.tobytes(), txn=self.dbtxn
+                        )
+                        # Why not use cursor_high throughout this method?
+                        # Then why not use DB_CURRENT and remove the delete()?
+                        cursor_high.delete()
                         cursor_new.put(
                             k,
                             b"".join(
@@ -269,7 +256,9 @@ class Database(_databasedu.Database):
                     else:
                         # self._path_marker.add('p5b-b')
                         self.segment_table[file].put(
-                            int.from_bytes(segref[-4:], "big"), seg.tobytes()
+                            int.from_bytes(segref[-4:], "big"),
+                            seg.tobytes(),
+                            txn=self.dbtxn,
                         )
                         cursor_high.delete()
                         cursor_high.put(
@@ -303,26 +292,9 @@ class Database(_databasedu.Database):
         if field not in self.value_segments[file]:
             return
 
-        # Lookup table is much quicker, and noticeable, in bulk use.
-        int_to_bytes = self._int_to_bytes
-
-        segvalues = self.value_segments[file][field]
-
         # Prepare to wrap the record numbers in an appropriate Segment class.
-        for k in segvalues:
-            value = segvalues[k]
-            if isinstance(value, list):
-                segvalues[k] = [
-                    len(value),
-                    b"".join([int_to_bytes[n] for n in value]),
-                ]
-            elif isinstance(value, Bitarray):
-                segvalues[k] = [
-                    value.count(),
-                    value.tobytes(),
-                ]
-            elif isinstance(value, int):
-                segvalues[k] = [1, value]
+        self._prepare_segment_record_list(file, field)
+        segvalues = self.value_segments[file][field]
 
         # New records go into temporary databases, one for each segment, except
         # when filling the segment which was high when this update started.
@@ -339,13 +311,14 @@ class Database(_databasedu.Database):
         # the indentation seems too far right for easy reading: there is an
         # extra 'try ... finally ...' compared with the _sqlitedu module which
         # makes the difference.)
+        # Below now obsolete.
         # Note the substantive difference between this module and _sqlitedu:
         # the code for Berkeley DB updates the main index directly if an entry
         # already exists, but the code for SQLite always updates a temporary
         # table and merges into the main table later.
-        cursor_new = self.table[SUBFILE_DELIMITER.join((file, field))][
-            -1
-        ].cursor(txn=self.dbtxn)
+        cursor_new = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
+            txn=self.dbtxn
+        )
         try:
             if (
                 self.high_segment[file] == segment
@@ -367,7 +340,9 @@ class Database(_databasedu.Database):
                 del segvalues[skey]
                 k = skey.encode()
                 if count > 1:
-                    srn = self.segment_table[file].append(records)
+                    srn = self.segment_table[file].append(
+                        records, txn=self.dbtxn
+                    )
                     cursor_new.put(
                         k,
                         b"".join(
@@ -401,192 +376,222 @@ class Database(_databasedu.Database):
         self.segment_table[file].sync()
 
     def new_deferred_root(self, file, field):
-        """Make new DB in dbenv for deferred updates and close current one."""
-        tablename = SUBFILE_DELIMITER.join((file, field))
-        self.table[tablename].append(self._dbe.DB(self.dbenv))
-        if len(self.table[tablename]) > 2:
-            try:
-                self.table[tablename][-2].close()
-            except:
-                pass
-        try:
-            # am = self.specification[file][FIELDS][field].get(ACCESS_METHOD)
-            self.table[tablename][-1].set_flags(self._dbe.DB_DUPSORT)
-            secondary = SUBFILE_DELIMITER.join(
-                (str(len(self.table[tablename]) - 1), file, field)
-            )
-            self.table[tablename][-1].open(
-                secondary if self.home_directory is not None else None,
-                secondary,
-                self._dbe.DB_BTREE,
-                self._dbe.DB_CREATE,
-                txn=self.dbtxn,
-            )
-        except:
-            for obj in self.table[tablename][1:]:
-                try:
-                    obj.close()
-                except:
-                    pass
-            self.close()
-            raise
+        """Do nothing.
+
+        Populating main database is slower than using a sequence of small
+        staging areas, but makes transaction commits in applications at
+        convenient intervals awkward.
+
+        Deferred update always uses the '-1' database so the main database is
+        accessed automatically since it is the '0' database.
+        """
 
     def merge(self, file, field):
-        """Merge the segment deferred updates into database."""
-        # Merge the segment deferred updates into database.
-
-        # Some of the unit testing using commented '_path_marker' code can be
-        # done with unittest.mock with suitable blocks of code delegated to
-        # methods.  For example is the outer 'finally' block executed with or
-        # without the block labelled 'p3'?  But how about the blocks labelled
-        # 'p13' through 'p18'?
-        # Dividing 'merge' into 'merge main' and some 'merge helpers' to avoid
-        # commented testing code seems wrong since doing discrete portions of
-        # merge is nonsense.
-        # To verify path coverage uncomment the '_path_marker' code.
-        # self._path_marker = set()
-
-        tablename = SUBFILE_DELIMITER.join((file, field))
-
-        # Any deferred updates?
-        if len(self.table[tablename]) == 1:
-            # self._path_marker.add('p1')
-            return
-
-        # self._path_marker.add('p2')
-        # Rename existing index and create new empty one.
-        # Open the old and new index, and all the deferred update indexes.
-        # am = self.specification[file][FIELDS][field].get(ACCESS_METHOD)
-        dudbc = len(self.table[tablename]) - 1
-        # if self._file_per_database:
-        #    f, d = self.table[tablename][0].get_dbname()
-        #    self.table[tablename][0].close()
-        #    newname = SUBFILE_DELIMITER.join(('0', d))
-        #    if self.home_directory is not None:
-        #        newname = os.path.join(self.home_directory, newname)
-        #    self.dbenv.dbrename(f, None, newname=newname)
-        #    self.dbenv.dbrename(
-        #        newname, d, newname=SUBFILE_DELIMITER.join(('0', d)))
-        #    self.table[tablename] = [self._dbe.DB(self.dbenv)]
-        #    self.table[tablename][0].set_flags(self._dbe.DB_DUPSORT)
-        #    self.table[tablename][0].open(
-        #        f,
-        #        dbname=d,
-        #        dbtype=self._dbe.DB_BTREE,
-        #        flags=self._dbe.DB_CREATE)
-        del self.table[tablename][1:]
-        for i in range(dudbc):
-            # self._path_marker.add('p3')
-            self.table[tablename].append(self._dbe.DB(self.dbenv))
-            self.table[tablename][-1].set_flags(self._dbe.DB_DUPSORT)
-            secondary = SUBFILE_DELIMITER.join(
-                (str(len(self.table[tablename]) - 1), file, field)
-            )
-            self.table[tablename][-1].open(
-                secondary if self.home_directory is not None else None,
-                dbname=secondary,
-                dbtype=self._dbe.DB_BTREE,
-            )
-        # if self._file_per_database:
-        #    self.table[tablename].insert(1, self._dbe.DB(self.dbenv))
-        #    self.table[tablename][1].set_flags(self._dbe.DB_DUPSORT)
-        #    self.table[tablename][1].open(
-        #        newname if self.home_directory is not None else None,
-        #        dbname=SUBFILE_DELIMITER.join(('0', d)),
-        #        dbtype=self._dbe.DB_BTREE)
-
-        # Write the entries from the old index and deferred update indexes to
-        # the new index in sort order: otherwise might as well have written the
-        # index entries direct to the old index rather than to the deferred
-        # update indexes.
-        # Assume at least 65536 records in each index. (segment_sort_scale)
-        # But OS ought to make the buffering done here a waste of time.
-        db_deferred = self.table[tablename][1:]
-        db_buffers = []
-        db_cursors = []
-        for dbo in db_deferred:
-            # self._path_marker.add('p4')
-            db_buffers.append(collections.deque())
-            db_cursors.append(dbo.cursor())
-        try:
-            length_limit = int(
-                SegmentSize.segment_sort_scale // max(1, len(db_buffers))
-            )
-            for i, buffer in enumerate(db_buffers):
-                # self._path_marker.add('p5')
-                dbc = db_cursors[i]
-                while len(buffer) < length_limit:
-                    # self._path_marker.add('p6')
-                    buffer.append(dbc.next())
-                try:
-                    # self._path_marker.add('p7')
-                    while buffer[-1] is None:
-                        # self._path_marker.add('p8')
-                        buffer.pop()
-                except IndexError:
-                    # self._path_marker.add('p9')
-                    dbc.close()
-                    db_cursors[i] = None
-                    fname, dname = db_deferred[i].get_dbname()
-                    db_deferred[i].close()
-                    # print('*', f, d)
-                    if fname is not None:
-                        self.dbenv.dbremove(fname)  # , database=d)
-                    del fname, dname
-                del buffer
-                del dbc
-            updates = []
-            heapq.heapify(updates)
-            heappop = heapq.heappop
-            heappush = heapq.heappush
-            for i, buffer in enumerate(db_buffers):
-                # self._path_marker.add('p10')
-                if buffer:
-                    # self._path_marker.add('p11')
-                    heappush(updates, (buffer.popleft(), i))
-            cursor = self.table[tablename][0].cursor()
-            try:
-                while updates:
-                    # self._path_marker.add('p12')
-                    record, i = heappop(updates)
-                    cursor.put(record[0], record[1], self._dbe.DB_KEYLAST)
-                    buffer = db_buffers[i]
-                    if not buffer:
-                        # self._path_marker.add('p13')
-                        dbc = db_cursors[i]
-                        if dbc is None:
-                            # self._path_marker.add('p14')
-                            continue
-                        while len(buffer) < length_limit:
-                            # self._path_marker.add('p15')
-                            buffer.append(dbc.next())
-                        try:
-                            # self._path_marker.add('p16')
-                            while buffer[-1] is None:
-                                # self._path_marker.add('p17')
-                                buffer.pop()
-                        except IndexError:
-                            # self._path_marker.add('p18')
-                            dbc.close()
-                            db_cursors[i] = None
-                            fname, dname = db_deferred[i].get_dbname()
-                            db_deferred[i].close()
-                            # print(f, d)
-                            if fname is not None:
-                                self.dbenv.dbremove(fname)  # , database=d)
-                            del fname, dname
-                            continue
-                        del dbc
-                    heappush(updates, (buffer.popleft(), i))
-            finally:
-                cursor.close()
-        finally:
-            for dbc in db_cursors:
-                # self._path_marker.add('p19')
-                if dbc:
-                    # self._path_marker.add('p20')
-                    dbc.close()
+        """Do nothing: there is nothing to do in _dbdu module."""
 
     def get_ebm_segment(self, ebm_control, key):
         """Return existence bitmap for segment number 'key'."""
-        return ebm_control.ebm_table.get(key, txn=self.dbtxn)
+        # record keys are 1-based but segment_numbers are 0-based.
+        return ebm_control.ebm_table.get(key + 1, txn=self.dbtxn)
+
+    def find_value_segments(self, field, file):
+        """Yield segment references for field in file."""
+        cursor = self.table[SUBFILE_DELIMITER.join((file, field))].cursor(
+            txn=self.dbtxn
+        )
+        try:
+            record = cursor.first()
+            while record:
+                value, segment = record
+                if len(segment) == 6:
+                    count = b"\x00\x01"
+                    reference = segment[4:]
+                else:
+                    assert len(segment) == 10
+                    count = segment[4:6]
+                    reference = segment[6:]
+                yield [
+                    value,
+                    segment[:4],
+                    EXISTING_SEGMENT_REFERENCE,
+                    count,
+                    reference,
+                ]
+                record = cursor.next()
+        finally:
+            cursor.close()
+
+    def delete_index(self, file, field):
+        """Remove all records from database for field in file."""
+        self.table[SUBFILE_DELIMITER.join((file, field))].truncate(
+            txn=self.dbtxn
+        )
+
+    def merge_writer(self, file, field):
+        """Return a Writer instance for the field index on table file.
+
+        Call the write() method with object returned by next_sorted_item
+        method.
+
+        """
+
+        def make_segment_from_high(high, transaction):
+            if high[2] == b"\x00\x01":
+                return RecordsetSegmentInt(
+                    int.from_bytes(high[1], byteorder="big"),
+                    None,
+                    records=high[3],
+                )
+            bytestring_records = segment_table.get(
+                int.from_bytes(high[3], byteorder="big"), txn=transaction
+            )
+            if len(bytestring_records) == SegmentSize.db_segment_size_bytes:
+                return RecordsetSegmentBitarray(
+                    int.from_bytes(high[1], byteorder="big"),
+                    None,
+                    records=bytestring_records,
+                )
+            return RecordsetSegmentList(
+                int.from_bytes(high[1], byteorder="big"),
+                None,
+                records=bytestring_records,
+            )
+
+        def make_segment_from_item(item):
+            if item[2] == b"\x00\x01":
+                return RecordsetSegmentInt(
+                    int.from_bytes(item[1], byteorder="big"),
+                    None,
+                    records=item[3],
+                )
+            if len(item[3]) == SegmentSize.db_segment_size_bytes:
+                return RecordsetSegmentBitarray(
+                    int.from_bytes(item[1], byteorder="big"),
+                    None,
+                    records=item[3],
+                )
+            return RecordsetSegmentList(
+                int.from_bytes(item[1], byteorder="big"),
+                None,
+                records=item[3],
+            )
+
+        def read_high_item_in_index(cursor):
+            record = cursor.last()
+            assert len(record) == 2
+            value, segment = record
+            if len(segment) == 6:
+                count = b"x00\x01"
+                reference = segment[4:]
+            else:
+                count = segment[4:6]
+                reference = segment[6:]
+            return [
+                value,
+                segment[:4],
+                count,
+                reference,
+            ]
+
+        keylast = self._dbe.DB_KEYLAST
+        table = self.table[SUBFILE_DELIMITER.join((file, field))]
+        segment_table = self.segment_table[file]
+
+        class Writer:
+            """Write index entries to database."""
+
+            def __init__(self, database):
+                self.prev_segment = None
+                self.prev_key = None
+                self.database = database
+                self.cursor = table.cursor(txn=self.database.dbtxn)
+
+            def make_new_cursor(self):
+                """Create a cursor on the assumed new transaction.
+
+                The existing cursor is retained for no transaction.
+
+                """
+                if self.database.dbtxn is None:
+                    return
+                self.cursor = table.cursor(txn=self.database.dbtxn)
+
+            def close_cursor(self):
+                """Close the cursor open on the index."""
+                self.cursor.close()
+
+            def write(self, item):
+                """Write item to index on database."""
+                assert len(item) == 5
+                segment = item[1]
+                if self.prev_segment != segment:
+                    self.prev_segment = segment
+                    self.prev_key = item[0]
+                    item_type = item.pop(2)
+                    if item_type == EXISTING_SEGMENT_REFERENCE:
+                        self.cursor.put(item[0], b"".join(item[1:]), keylast)
+                        assert len(item) == 4
+                        return
+                    if int.from_bytes(item[-2], byteorder="big") > 1:
+                        item[-1] = segment_table.append(
+                            item[-1], txn=self.database.dbtxn
+                        ).to_bytes(4, byteorder="big")
+                        assert len(item) == 4
+                    else:
+                        item.pop(2)
+                        assert len(item) == 3
+                    self.cursor.put(item[0], b"".join(item[1:]), keylast)
+                    assert item_type == NEW_SEGMENT_CONTENT
+                    return
+                if self.prev_key == item[0]:
+                    assert item[2] == NEW_SEGMENT_CONTENT
+                    del item[2]
+                    high = read_high_item_in_index(self.cursor)
+                    existing_segment = make_segment_from_high(
+                        high, self.database.dbtxn
+                    )
+                    new_segment = make_segment_from_item(item)
+                    new_segment |= existing_segment
+                    new_segment.normalize()
+                    item[
+                        -2
+                    ] = self.database.encode_number_for_sequential_file_dump(
+                        new_segment.count_records(), 2
+                    )
+                    if int.from_bytes(high[-2], byteorder="big") == 1:
+                        item[-1] = segment_table.append(
+                            new_segment.tobytes(), txn=self.database.dbtxn
+                        ).to_bytes(4, byteorder="big")
+                        self.cursor.delete()
+                        self.cursor.put(item[0], b"".join(item[1:]), keylast)
+                    else:
+                        segment_table.put(
+                            int.from_bytes(high[-1], "big"),
+                            new_segment.tobytes(),
+                            txn=self.database.dbtxn,
+                        )
+                        self.cursor.delete()
+                        item[-1] = high[-1]
+                        self.cursor.put(item[0], b"".join(item[1:]), keylast)
+                    assert len(item) == 4
+                    return
+                item_type = item.pop(2)
+                if item_type == EXISTING_SEGMENT_REFERENCE:
+                    self.prev_key = item[0]
+                    self.cursor.put(item[0], b"".join(item[1:]), keylast)
+                    assert len(item) == 4
+                    return
+                if int.from_bytes(item[-2], byteorder="big") > 1:
+                    item[-1] = segment_table.append(
+                        item[-1],
+                        txn=self.database.dbtxn,
+                    ).to_bytes(4, byteorder="big")
+                    assert len(item) == 4
+                else:
+                    item.pop(2)
+                    assert len(item) == 3
+                self.cursor.put(item[0], b"".join(item[1:]), keylast)
+                assert item_type == NEW_SEGMENT_CONTENT
+                return
+
+        return Writer(self)
