@@ -7,21 +7,24 @@
 When using sqlite3 the Python version must be 3.6 or later.
 
 """
-import heapq
-import collections
-
-from .bytebit import Bitarray
 from .constants import (
     SECONDARY,
     SQLITE_SEGMENT_COLUMN,
     SQLITE_COUNT_COLUMN,
     SQLITE_VALUE_COLUMN,
     SUBFILE_DELIMITER,
+    EXISTING_SEGMENT_REFERENCE,
     INDEXPREFIX,
-    TABLEPREFIX,
+    NEW_SEGMENT_CONTENT,
+    SQLITE_RECORDS_COLUMN,
 )
 from .segmentsize import SegmentSize
 from . import _databasedu
+from .recordset import (
+    RecordsetSegmentBitarray,
+    RecordsetSegmentInt,
+    RecordsetSegmentList,
+)
 
 
 class DatabaseError(Exception):
@@ -59,7 +62,7 @@ class Database(_databasedu.Database):
     # def commit(self):
     #    """Override superclass method to do nothing."""
 
-    def database_cursor(self, file, field, keyrange=None):
+    def database_cursor(self, file, field, keyrange=None, recordset=None):
         """Not implemented for deferred update."""
         raise DatabaseError("database_cursor not implemented")
 
@@ -72,7 +75,7 @@ class Database(_databasedu.Database):
                     "select",
                     file,
                     "from",
-                    self.table[file][0],
+                    self.table[file],
                     "order by",
                     file,
                     "desc",
@@ -92,7 +95,7 @@ class Database(_databasedu.Database):
                 continue
             finally:
                 cursor.close()
-            self.write_existence_bit_map(file, segment)
+            self._write_existence_bit_map(file, segment)
             for secondary in self.specification[file][SECONDARY]:
                 self.sort_and_write(file, secondary, segment)
                 self.merge(file, secondary)
@@ -110,10 +113,7 @@ class Database(_databasedu.Database):
         # Timings when adding to an empty database suggest the sqlite3 version
         # would be a little slower than the bsddb3 version.
 
-        self._int_to_bytes = [
-            n.to_bytes(2, byteorder="big")
-            for n in range(SegmentSize.db_segment_size)
-        ]
+        self.set_int_to_bytes_lookup(lookup=True)
 
         # Comment these if the 'do-nothing' override of commit() is commented.
         # self.dbenv.cursor().execute('pragma journal_mode = off')
@@ -145,7 +145,7 @@ class Database(_databasedu.Database):
 
     def unset_defer_update(self):
         """Tidy-up at end of deferred update run."""
-        self._int_to_bytes = None
+        self.set_int_to_bytes_lookup(lookup=False)
         for file in self.specification:
             self.high_segment[file] = None
             self.first_chunk[file] = None
@@ -158,7 +158,7 @@ class Database(_databasedu.Database):
         # self.dbenv.cursor().execute('pragma journal_mode = delete')
         # self.dbenv.cursor().execute('pragma synchronous = full')
 
-    def write_existence_bit_map(self, file, segment):
+    def _write_existence_bit_map(self, file, segment):
         """Write the existence bit map for segment in file."""
         assert file in self.specification
         statement = " ".join(
@@ -195,26 +195,9 @@ class Database(_databasedu.Database):
         if field not in self.value_segments[file]:
             return
 
-        # Lookup table is much quicker, and noticeable, in bulk use.
-        int_to_bytes = self._int_to_bytes
-
-        segvalues = self.value_segments[file][field]
-
         # Prepare to wrap the record numbers in an appropriate Segment class.
-        for k in segvalues:
-            svk = segvalues[k]
-            if isinstance(svk, list):
-                segvalues[k] = [
-                    len(svk),
-                    b"".join([int_to_bytes[n] for n in svk]),
-                ]
-            elif isinstance(svk, Bitarray):
-                segvalues[k] = [
-                    svk.count(),
-                    svk.tobytes(),
-                ]
-            elif isinstance(svk, int):
-                segvalues[k] = [1, svk]
+        self._prepare_segment_record_list(file, field)
+        segvalues = self.value_segments[file][field]
 
         # New records go into temporary databases, one for each segment, except
         # when filling the segment which was high when this update started.
@@ -227,14 +210,13 @@ class Database(_databasedu.Database):
         # The low segment in the import may have to be merged with an existing
         # high segment on the database, or the current segment in the import
         # may be done in chunks of less than a complete segment.
-        # Note the difference between this code, and the similar code in module
-        # apswduapi.py, and the code in module dbduapi.py: the Berkeley DB
-        # code updates the main index directly if an entry already exists, but
-        # the Sqlite code always updates a temporary table and merges into the
-        # main table later.
-        tablename = self.table[SUBFILE_DELIMITER.join((file, field))][-1]
+        # Below now obsolete.
+        # Note the substantive difference between this module and _dbdu:
+        # the code for Berkeley DB updates the main index directly if an entry
+        # already exists, but the code for SQLite always updates a temporary
+        # table and merges into the main table later.
+        tablename = self.table[SUBFILE_DELIMITER.join((file, field))]
         if self.high_segment[file] == segment or not self.first_chunk[file]:
-
             # select (index value, segment number, record count, key reference)
             # statement for (index value, segment number).  Execution returns
             # None if no splicing needed.
@@ -347,7 +329,7 @@ class Database(_databasedu.Database):
         # statement.
         insert_new_segment = " ".join(
             (
-                "insert into",
+                "insert or replace into",
                 tablename,
                 "(",
                 field,
@@ -375,218 +357,352 @@ class Database(_databasedu.Database):
             yield (k, segment, svk[0], svk[1])
 
     def new_deferred_root(self, file, field):
-        """Make new temporary table for deferred updates and close current."""
-        # The temporary tables go in /tmp, at least in OpenBSD where the
-        # default mount points allocate far too little space to /tmp for this
-        # program.
-        # The FreeBSD default layout is now a single '/' area, so the space is
-        # available but it is not clear which '/<any>' gets used.
-        tablename = SUBFILE_DELIMITER.join((file, field))
-        self.table[tablename].append(
-            SUBFILE_DELIMITER.join(
-                (TABLEPREFIX, str(len(self.table[tablename]) - 1), tablename)
-            )
-        )
-        self.index[tablename].append(
-            "".join((INDEXPREFIX, self.table[tablename][-1]))
-        )
-        try:
-            statement = " ".join(
-                (
-                    "create temp table",
-                    self.table[tablename][-1],
-                    "(",
-                    field,
-                    ",",
-                    SQLITE_SEGMENT_COLUMN,
-                    ",",
-                    SQLITE_COUNT_COLUMN,
-                    ",",
-                    file,
-                    ")",
-                )
-            )
-            cursor = self.dbenv.cursor()
-            try:
-                cursor.execute(statement)
-            finally:
-                cursor.close()
+        """Do nothing.
 
-            # The index is not needed if deferred_update_points has exactly one
-            # element because each segment is done in one chunk.
-            statement = " ".join(
-                (
-                    "create unique index",
-                    self.index[tablename][-1],
-                    "on",
-                    self.table[tablename][-1],
-                    "(",
-                    field,
-                    ",",
-                    SQLITE_SEGMENT_COLUMN,
-                    ")",
-                )
-            )
-            cursor = self.dbenv.cursor()
-            try:
-                cursor.execute(statement)
-            finally:
-                cursor.close()
+        Populating main database is slower than using a sequence of small
+        staging areas, but makes transaction commits in applications at
+        convenient intervals awkward.
 
-        except:
-            self.close()
-            raise
+        Deferred update always uses the '-1' database so the main database is
+        accessed automatically since it is the '0' database.
+        """
 
     def merge(self, file, field):
-        """Merge deferred updates for field in file into database."""
-        # Some of the unit testing using commented '_path_marker' code can be
-        # done with unittest.mock with suitable blocks of code delegated to
-        # methods.  For example is the outer 'finally' block executed with or
-        # without the block labelled 'p3'?  But how about the blocks labelled
-        # 'p13' through 'p18'?
-        # Dividing 'merge' into 'merge main' and some 'merge helpers' to avoid
-        # commented testing code seems wrong since doing discrete portions of
-        # merge is nonsense.
-        # To verify path coverage uncomment the '_path_marker' code.
-        # self._path_marker = set()
-
-        # Any deferred updates?
-        if len(self.table[SUBFILE_DELIMITER.join((file, field))]) == 1:
-            # self._path_marker.add('p1')
-            return
-
-        # Cannot imitate the renaming of databases done in bsddb3.  The sqlite3
-        # table names can be changed using 'alter', but the indicies keep their
-        # names. A bsddb3 database is equivalent to an sqlite3 table and it's
-        # index here.
-
-        # Write the entries from the deferred update indices to the existing
-        # index in sort order: otherwise might as well have written the index
-        # entries direct to the existing index rather than to the deferred
-        # update indices.
-        # Each deferred update index will have SegmentSize.segment_sort_scale
-        # records at most.
-
-        # self._path_marker.add('p2')
-        sq_deferred = self.table[SUBFILE_DELIMITER.join((file, field))][1:]
-        sq_buffers = []
-        sq_cursors = []
-        try:
-            for i, sqo in enumerate(sq_deferred):
-                # self._path_marker.add('p3')
-
-                # The 'order by' clause is not needed if deferred_update_points
-                # has exactly one element because each segment is done in one
-                # chunk.
-                select_segments = " ".join(
-                    (
-                        "select",
-                        field,
-                        ",",
-                        SQLITE_SEGMENT_COLUMN,
-                        ",",
-                        SQLITE_COUNT_COLUMN,
-                        ",",
-                        file,
-                        "from",
-                        sqo,
-                        "order by",
-                        field,
-                        ",",
-                        SQLITE_SEGMENT_COLUMN,
-                    )
-                )
-
-                sq_cursors.append(self.dbenv.cursor().execute(select_segments))
-
-            # The arraysize property is assumed available only in sqlite3.
-            # In particular, apsw does not have it.
-            arraysize = int(
-                SegmentSize.segment_sort_scale // max(1, len(sq_cursors))
-            )
-
-            # self._path_marker.add('p4')
-            for i, sqc in enumerate(sq_cursors):
-                # self._path_marker.add('p5')
-                buffer = collections.deque()
-                sq_buffers.append(buffer)
-                for j in range(arraysize):
-                    # self._path_marker.add('p6')
-                    row = sqc.fetchone()
-                    if row is None:
-                        # self._path_marker.add('p7')
-                        break
-                    buffer.append(row)
-                if len(buffer) < arraysize:
-                    # self._path_marker.add('p8')
-                    sq_cursors[i].close()
-                    sq_cursors[i] = None
-                del buffer
-            updates = []
-            heapq.heapify(updates)
-            heappop = heapq.heappop
-            heappush = heapq.heappush
-            for i, buffer in enumerate(sq_buffers):
-                # self._path_marker.add('p9')
-                if buffer:
-                    # self._path_marker.add('p10')
-                    heappush(updates, (buffer.popleft(), i))
-            insert_new_segment = " ".join(
-                (
-                    "insert into",
-                    self.table[SUBFILE_DELIMITER.join((file, field))][0],
-                    "(",
-                    field,
-                    ",",
-                    SQLITE_SEGMENT_COLUMN,
-                    ",",
-                    SQLITE_COUNT_COLUMN,
-                    ",",
-                    file,
-                    ")",
-                    "values ( ? , ? , ? , ? )",
-                )
-            )
-            cursor = self.dbenv.cursor()
-            try:
-                # self._path_marker.add('p11')
-                while updates:
-                    # self._path_marker.add('p12')
-                    record, i = heappop(updates)
-                    cursor.execute(insert_new_segment, record)
-                    buffer = sq_buffers[i]
-                    if not buffer:
-                        # self._path_marker.add('p13')
-                        sqc = sq_cursors[i]
-                        if sqc is None:
-                            # self._path_marker.add('p14')
-                            continue
-                        for j in range(arraysize):
-                            # self._path_marker.add('p15')
-                            row = sqc.fetchone()
-                            if row is None:
-                                # self._path_marker.add('p16')
-                                break
-                            buffer.append(row)
-                        if len(buffer) < arraysize:
-                            # self._path_marker.add('p17')
-                            sqc.close()
-                            sq_cursors[i] = None
-                        del sqc
-                        if not buffer:
-                            # self._path_marker.add('p18')
-                            continue
-                    heappush(updates, (buffer.popleft(), i))
-            finally:
-                cursor.close()
-        finally:
-            for sqc in sq_cursors:
-                # self._path_marker.add('p19')
-                if sqc:
-                    # self._path_marker.add('p20')
-                    sqc.close()
-        # self._path_marker.add('p21')
+        """Do nothing: there is nothing to do in _sqlitedu module."""
 
     def get_ebm_segment(self, ebm_control, key):
         """Return existence bitmap for segment number 'key'."""
         return ebm_control.get_ebm_segment(key, self.dbenv)
+
+    def find_value_segments(self, field, file):
+        """Yield segment references for field in file."""
+        statement = " ".join(
+            (
+                "select",
+                field,
+                ",",
+                SQLITE_SEGMENT_COLUMN,
+                ",",
+                SQLITE_COUNT_COLUMN,
+                ",",
+                file,
+                "from",
+                self.table[SUBFILE_DELIMITER.join((file, field))],
+                "order by",
+                field,
+                ",",
+                "Segment",
+            )
+        )
+        cursor = self.dbenv.cursor()
+        try:
+            for value, segment, count, reference in cursor.execute(
+                statement, ()
+            ):
+                yield [
+                    value,
+                    segment,
+                    EXISTING_SEGMENT_REFERENCE,
+                    count,
+                    reference,
+                ]
+        finally:
+            cursor.close()
+
+    def encode_number_for_sequential_file_dump(self, number, bytes_):
+        """Return number.
+
+        Argument bytes_ must be supplied for compatibility with database
+        engines which convert the number to a fixed-length bytestring
+        encoding.
+
+        """
+        del bytes_
+        return number
+
+    def encode_segment_for_sequential_file_dump(self, record_numbers):
+        """Return encoding of record numbers appropriate to record count."""
+        if len(record_numbers) > SegmentSize.db_upper_conversion_limit:
+            seg = SegmentSize.empty_bitarray.copy()
+            for bit in record_numbers:
+                seg[bit] = True
+            return seg.tobytes()
+        if len(record_numbers) > 1:
+            int_to_bytes = self._int_to_bytes
+            return b"".join([int_to_bytes[n] for n in record_numbers])
+        return record_numbers[0]
+
+    def delete_index(self, file, field):
+        """Remove all records from database for field in file.
+
+        Implemented by dropping table followed by creating it and it's
+        indicies.
+
+        """
+        statement = " ".join(
+            (
+                "drop table if exists",
+                self.table[SUBFILE_DELIMITER.join((file, field))],
+            )
+        )
+        cursor = self.dbenv.cursor()
+        try:
+            cursor.execute(statement)
+        finally:
+            cursor.close()
+        secondary = SUBFILE_DELIMITER.join((file, field))
+        statement = " ".join(
+            (
+                "create table if not exists",
+                secondary,
+                "(",
+                field,
+                ",",
+                SQLITE_SEGMENT_COLUMN,
+                ",",
+                SQLITE_COUNT_COLUMN,
+                ",",
+                file,
+                ")",
+            )
+        )
+        cursor = self.dbenv.cursor()
+        try:
+            cursor.execute(statement)
+        finally:
+            cursor.close()
+        indexname = "".join(
+            (INDEXPREFIX, SUBFILE_DELIMITER.join((file, field)))
+        )
+        statement = " ".join(
+            (
+                "create unique index if not exists",
+                indexname,
+                "on",
+                secondary,
+                "(",
+                field,
+                ",",
+                SQLITE_SEGMENT_COLUMN,
+                ")",
+            )
+        )
+        cursor = self.dbenv.cursor()
+        try:
+            cursor.execute(statement)
+        finally:
+            cursor.close()
+
+    def merge_writer(self, file, field):
+        """Return a Writer instance for the field index on table file.
+
+        Call the write() method with object returned by next_sorted_item
+        method.
+
+        """
+
+        def make_segment_from_item(item):
+            if item[2] == 1:
+                return RecordsetSegmentInt(
+                    item[1],
+                    None,
+                    records=item[3].to_bytes(2, byteorder="big"),
+                )
+            if len(item[3]) == SegmentSize.db_segment_size_bytes:
+                return RecordsetSegmentBitarray(item[1], None, records=item[3])
+            return RecordsetSegmentList(item[1], None, records=item[3])
+
+        assert file != field
+        secondary = self.table[SUBFILE_DELIMITER.join((file, field))]
+        write_item_to_index = " ".join(
+            (
+                "insert into",
+                secondary,
+                "(",
+                field,
+                ",",
+                SQLITE_SEGMENT_COLUMN,
+                ",",
+                SQLITE_COUNT_COLUMN,
+                ",",
+                file,
+                ")",
+                "values ( ? , ? , ? , ? )",
+            )
+        )
+        replace_index_item = " ".join(
+            (
+                "update",
+                secondary,
+                "set",
+                SQLITE_COUNT_COLUMN,
+                "= ? ,",
+                file,
+                "= ?",
+                "where",
+                field,
+                "== ? and",
+                SQLITE_SEGMENT_COLUMN,
+                "== ?",
+            )
+        )
+        replace_index_count = " ".join(
+            (
+                "update",
+                secondary,
+                "set",
+                SQLITE_COUNT_COLUMN,
+                "= ?",
+                "where",
+                field,
+                "== ? and",
+                SQLITE_SEGMENT_COLUMN,
+                "== ?",
+            )
+        )
+        read_high_item_in_index = " ".join(
+            (
+                "select",
+                field,
+                ",",
+                SQLITE_SEGMENT_COLUMN,
+                ",",
+                SQLITE_COUNT_COLUMN,
+                ",",
+                file,
+                "from",
+                secondary,
+                "order by",
+                field,
+                "desc",
+                ",",
+                SQLITE_SEGMENT_COLUMN,
+                "desc",
+                "limit 1",
+            )
+        )
+        insert_segment_records = " ".join(
+            (
+                "insert into",
+                self.segment_table[file],
+                "(",
+                SQLITE_RECORDS_COLUMN,
+                ")",
+                "values ( ? )",
+            )
+        )
+        last_inserted_row = " ".join(
+            (
+                "select last_insert_rowid() from",
+                self.segment_table[file],
+            )
+        )
+        set_segment_records = " ".join(
+            (
+                "update",
+                self.segment_table[file],
+                "set",
+                SQLITE_RECORDS_COLUMN,
+                "= ?",
+                "where rowid == ?",
+            )
+        )
+
+        class Writer:
+            """Write index entries to database."""
+
+            def __init__(self, database):
+                self.prev_segment = None
+                self.prev_key = None
+                self.database = database
+                self.cursor = database.dbenv.cursor()
+
+            def make_new_cursor(self):
+                """Do nothing.
+
+                Present for compatibility with _dbdu, _dbdu_tkinter, and
+                _lmdbdu.
+                """
+
+            def close_cursor(self):
+                """Do nothing.
+
+                Present for compatibility with _dbdu, _dbdu_tkinter, and
+                _lmdbdu.
+                """
+
+            def write(self, item):
+                """Write item to index on database."""
+                assert len(item) == 5
+                segment = item[1]
+                if self.prev_segment != segment:
+                    self.prev_segment = segment
+                    self.prev_key = item[0]
+                    item_type = item.pop(2)
+                    if item_type == EXISTING_SEGMENT_REFERENCE:
+                        self.cursor.execute(write_item_to_index, item)
+                        assert len(item) == 4
+                        return
+                    if item[-2] > 1:
+                        self.cursor.execute(
+                            insert_segment_records, (item[-1],)
+                        )
+                        item[-1] = self.cursor.execute(
+                            last_inserted_row
+                        ).fetchone()[0]
+                    self.cursor.execute(write_item_to_index, item)
+                    assert item_type == NEW_SEGMENT_CONTENT
+                    assert len(item) == 4
+                    return
+                if self.prev_key == item[0]:
+                    assert item[2] == NEW_SEGMENT_CONTENT
+                    del item[2]
+                    high = self.cursor.execute(
+                        read_high_item_in_index
+                    ).fetchone()
+                    new_segment = make_segment_from_item(item)
+                    new_segment |= self.database.populate_segment(high, file)
+                    new_segment.normalize()
+                    if high[2] == 1:
+                        self.cursor.execute(
+                            insert_segment_records, (new_segment.tobytes(),)
+                        )
+                        self.cursor.execute(
+                            replace_index_item,
+                            (
+                                new_segment.count_records(),
+                                self.cursor.execute(
+                                    last_inserted_row
+                                ).fetchone()[0],
+                                high[0],
+                                high[1],
+                            ),
+                        )
+                    else:
+                        self.cursor.execute(
+                            set_segment_records,
+                            (new_segment.tobytes(), high[3]),
+                        )
+                        self.cursor.execute(
+                            replace_index_count,
+                            (new_segment.count_records(), high[0], high[1]),
+                        )
+                    assert len(item) == 4
+                    return
+                item_type = item.pop(2)
+                if item_type == EXISTING_SEGMENT_REFERENCE:
+                    self.prev_key = item[0]
+                    self.cursor.execute(write_item_to_index, item)
+                    assert len(item) == 4
+                    return
+                if item[2] > 1:
+                    self.cursor.execute(insert_segment_records, (item[3],))
+                    item[3] = self.cursor.execute(
+                        last_inserted_row
+                    ).fetchone()[0]
+                self.cursor.execute(write_item_to_index, item)
+                assert item_type == NEW_SEGMENT_CONTENT
+                assert len(item) == 4
+                return
+
+        return Writer(self)
