@@ -47,9 +47,7 @@ class Tree:
         self.file = file
         self.field = field
         self.database = database
-        self.key_root = database.table[SUBFILE_DELIMITER.join((file, field))][
-            0
-        ]
+        self.key_root = database.table[SUBFILE_DELIMITER.join((file, field))]
         self.key_segment = SUBFILE_DELIMITER.join(
             (self.key_root, SEGMENT_KEY_SUFFIX)
         )
@@ -519,6 +517,7 @@ class Tree:
         node.modified = True
 
     def _merge_leaf(self, key, nodepath, deletion_point):
+        del key
         node = nodepath[-1]
         parent = nodepath[-2]
         entries = parent.node[_Node.ENTRIES]
@@ -899,7 +898,6 @@ class Cursor:
 
 
 class _Node:
-
     # The valid values of node[NODE_TYPE]
     ROOT = 1
     SOLO_ROOT = 2
@@ -942,6 +940,7 @@ class _Node:
         return len(self.node[_Node.KEYS])
 
     def insert_into_branch_or_root(self, key, node_number):
+        """Insert key into node node_number in sorted order."""
         keys = self.node[_Node.KEYS]
         insertion_point = bisect_left(keys, key)
         assert insertion_point == len(keys) or key != keys[insertion_point]
