@@ -11,9 +11,10 @@ Prefer to use the gnu_database module normally.
 """
 from . import gnu_database
 from .core import _nosqldu
+from .core import _nosql
 
 
-class Database(_nosqldu.Database, gnu_database.Database):
+class Database(gnu_database.Database, _nosqldu.Database, _nosql.Database):
     """Define deferred update Database class using gnu_database module.
 
     Deferred update behaviour comes from the _nosqldu.Database class.
