@@ -11,9 +11,12 @@ Prefer to use the sqlite3_database module normally.
 """
 from . import sqlite3_database
 from .core import _sqlitedu
+from .core import _sqlite
 
 
-class Database(_sqlitedu.Database, sqlite3_database.Database):
+class Database(
+    sqlite3_database.Database, _sqlitedu.Database, _sqlite.Database
+):
     """Define deferred update Database class using sqlite3 module.
 
     Deferred update behaviour comes from the _sqlitedu.Database class.
