@@ -45,10 +45,21 @@ try:
     import berkeleydb
 except ImportError:  # Not ModuleNotFoundError for Pythons earlier than 3.6
     berkeleydb = None
+try:
+    import lmdb
+except ImportError:  # Not ModuleNotFoundError for Pythons earlier than 3.6
+    lmdb = None
 try:
     import bsddb3
 except ImportError:  # Not ModuleNotFoundError for Pythons earlier than 3.6
     bsddb3 = None
+try:
+    from .. import db_tcl
+
+    if db_tcl.tcl_tk_call is None:
+        db_tcl = None
+except ImportError:  # Not ModuleNotFoundError for Pythons earlier than 3.6
+    db_tcl = None
 try:
     from dptdb import dptapi
 except ImportError:  # Not ModuleNotFoundError for Pythons earlier than 3.6
@@ -133,6 +144,8 @@ class CreateDatabase:
                 dptapi,
                 berkeleydb,
                 bsddb3,
+                db_tcl,
+                lmdb,
                 vedis,
                 unqlite,
                 apsw,
@@ -167,7 +180,6 @@ class CreateDatabase:
         text.pack(fill=tkinter.BOTH)
         self.menu = tkinter.Menu(master=frame, tearoff=False)
         self.__menu = self.menu
-        self.__xy = None
         self.root = root
         self.text = text
         self.entry = entry
@@ -191,7 +203,6 @@ class CreateDatabase:
                 )
             )
         entry.focus_set()
-        self.__help = None
 
     def insert_text(self, text):
         """Wrap Text widget insert with Enable and Disable state configure."""
@@ -213,11 +224,11 @@ class CreateDatabase:
     def show_menu(self, event=None):
         """Show the popup menu for widget."""
         self.__menu.tk_popup(*event.widget.winfo_pointerxy())
-        self.__xy = event.x, event.y
         self.__menu = self.menu
 
     def select_database_file(self, event=None):
         """Select a directory."""
+        del event
         localfilename = tkinter.filedialog.askdirectory(
             parent=self.text,
             title="Select directory where database is to be created",
@@ -236,7 +247,16 @@ class CreateDatabase:
         """
         engine = self.engine_list[int(self.database.get())]
         engine_database_class = self.engines[engine]
-        ssb = int(self.segmentsizebytes.get())
+
+        # ssb was not validated if database engine is dptapi because the
+        # value is ignored.
+        ssb = None
+        try:
+            ssb = int(self.segmentsizebytes.get())
+        except ValueError:
+            if dptapi is not engine:
+                raise
+
         if ssb == SEGMENT_SIZE_BYTES_FOR_TESTS:
             ssb = None
         path = self.directory.get()
@@ -251,6 +271,7 @@ class CreateDatabase:
 
     def create_folder_and_database(self, event=None):
         """Create database after creating folder if it does not exist."""
+        del event
         if self.directory.get() == "":
             tkinter.messagebox.showerror(
                 master=self.root, message="Please select a directory"
@@ -298,7 +319,7 @@ class CreateDatabase:
         if not os.path.exists(path):
             try:
                 os.mkdir(path)
-            except:
+            except Exception:
                 msg = (
                     "Cannot create directory\n",
                     os.path.basename(self.directory.get()),
@@ -416,7 +437,6 @@ class CreateDatabase:
 
 
 if __name__ == "__main__":
-
     if unqlite:
         try:
             from .. import unqlite_database
@@ -447,6 +467,13 @@ if __name__ == "__main__":
             apsw_database = None
     else:
         apsw_database = None
+    if lmdb:
+        try:
+            from .. import lmdb_database
+        except ImportError:
+            lmdb_database = None
+    else:
+        lmdb_database = None
     if berkeleydb:
         try:
             from .. import berkeleydb_database
@@ -461,6 +488,13 @@ if __name__ == "__main__":
             bsddb3_database = None
     else:
         bsddb3_database = None
+    if db_tcl:
+        try:
+            from .. import db_tkinter_database
+        except ImportError:
+            db_tkinter_database = None
+    else:
+        db_tkinter_database = None
     if dptapi:
         try:
             from .. import dpt_database
@@ -491,10 +525,14 @@ if __name__ == "__main__":
             engines = {}
             if dptapi:
                 engines[dptapi] = dpt_database.Database
+            if lmdb:
+                engines[lmdb] = lmdb_database.Database
             if berkeleydb:
                 engines[berkeleydb] = berkeleydb_database.Database
             if bsddb3:
                 engines[bsddb3] = bsddb3_database.Database
+            if db_tcl:
+                engines[db_tcl] = db_tkinter_database.Database
             if vedis:
                 engines[vedis] = vedis_database.Database
             if unqlite:
