@@ -11,9 +11,10 @@ Prefer to use the unqlite_database module normally.
 """
 from . import unqlite_database
 from .core import _nosqldu
+from .core import _nosql
 
 
-class Database(_nosqldu.Database, unqlite_database.Database):
+class Database(unqlite_database.Database, _nosqldu.Database, _nosql.Database):
     """Define deferred update Database class using unqlite_database module.
 
     Deferred update behaviour comes from the _nosqldu.Database class.
